#pragma once

/** ----------------------------------------------------------------------------
    Macro checks if some specified interface function is available and
    may be called. "Interface" here is a structure in memory that has
    the member ~size~ that defines the length of the structure body in bytes
    and one or more members that are pointers to interface functions.
      Interface function is regarded to be available if interface object
    is not NULL, members's offset within the structure is less than the
    ~size~ (taking into account the size of the pointer value) and is
    the member itself is not NULL.

    Param iface:
        pointer to the "interface" structure, may be NULL.
        The following structures are compatible with this macro:
          ~ElmTypeInfo~ -- MiniCAD element type definition;
          ~CadFileInterface~ -- CAD file type definition.

    Param funcName:
        valid literall name of the interface member function.

    Rets TRUE  if function can be called,
    Rets FALSE if function could not be called.
 */
#define IsIfaceFuncAvailable( iface, funcName )                    \
                ( (iface)                                          \
                  && ( ( (char *)&(iface)->funcName -              \
                         (char *)(iface) + sizeof( void * ) )      \
                         <= (uint)(iface)->size )                  \
                  && ( (iface)->funcName )                         \
                )



/**
    This is the common object destructor function prototype. In the VMT
    (Virtual Methods Table, aka ~iface~) it should be at the first place.

    The same prototype is used for ~GuiDC~, ~GuiWindow~, ~GuiBitmap~,
    ~GuiProvider~ and other objects.

    Use ~ObjFree()~ macro to directly call this function,
    or use ~AppCallObjFree()~ macro to free any object in
    the instance of its own MiniApp (called via AppCallHook()).

    @param[in] pointer to the object being freed
 */
typedef void ( INTERFACECALL Obj_Free )(
    void              *self
);
#define  ObjFree( obj )   (( *(**(Obj_Free ***)(obj)) )( (void *)(obj) ))
#define  AppCallObjFree( obj )    AppCallHook( (**(Obj_Free ***)(obj)), PSZ_PTR, (void *)(obj) )


